var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import * as shopAPI from './shopify.api.js';
import * as shopUtils from './shopify.utils.js';
import exceptionalError from 'exceptionalhandler';
import * as shopConstants from './shopify.constants.js';
import * as shopConverter from './shopify.conversions.js';
// GET ORDERS
export function getOrders(lookupFilters) {
    return __awaiter(this, void 0, void 0, function* () {
        if (shopUtils.isValEmpty(lookupFilters)) {
            throw new exceptionalError("Order type is required");
        }
        let statusLookup;
        switch (lookupFilters) {
            case "new":
                statusLookup = shopConstants.orderStatusOpen;
                break;
            default:
                throw new exceptionalError(`Invalid order type ${lookupFilters}`);
        }
        let orderLookup = yield shopAPI.shopify.rest.Order.all({
            session: shopAPI.shopSession,
            status: statusLookup
        });
        let returnOrders = [];
        for (const shopOrder of orderLookup.data) {
            //@ts-ignore breaking the law, breaking the law
            returnOrders.push(shopConverter.shopOrderToNiceOrder(shopOrder));
        }
        return returnOrders;
    });
}
// CHANGE ORDERS
export function shipOrder(orderId, trackingDetails, notifyUser = true) {
    return __awaiter(this, void 0, void 0, function* () {
        let fulfillmentIds = yield getOrderFulfillmentIDs(orderId);
        let ids = [];
        for (const f of fulfillmentIds) {
            if (!ids.includes(f.fulfillmentOrderId)) {
                ids.push(f.fulfillmentOrderId);
            }
        }
        yield addOrderTrackingDetails(ids, trackingDetails, notifyUser);
        yield closeOrder(orderId);
    });
}
export function getOrderFulfillmentIDs(orderId) {
    return __awaiter(this, void 0, void 0, function* () {
        let idMapping = [];
        let getOrderDeets = yield shopAPI.shopify.rest.FulfillmentOrder.all({
            session: shopAPI.shopSession,
            order_id: Number(orderId),
        });
        if (getOrderDeets.data.length === 0) {
            throw new exceptionalError(`No match for orderID ${orderId}`);
        }
        //@ts-ignore - more lazy shopify devs
        for (const lineItem of getOrderDeets.data[0].line_items) {
            idMapping.push({
                lineItemId: String(lineItem.line_item_id),
                fulfillmentOrderId: String(lineItem.fulfillment_order_id)
            });
        }
        return idMapping;
    });
}
export function addOrderTrackingDetails(fulfillmentOrderIds, trackingDetails, notifyUser = true) {
    return __awaiter(this, void 0, void 0, function* () {
        const fulfillmentUpdate = new shopAPI.shopify.rest.Fulfillment({ session: shopAPI.shopSession });
        if (!Array.isArray(fulfillmentOrderIds) || shopUtils.isValEmpty(fulfillmentOrderIds)) {
            throw new exceptionalError(`fulfillmentOrderIds must be an array with at least one fulfillment order ID`);
        }
        for (const fulfillmentOrderId of fulfillmentOrderIds) {
            if (typeof fulfillmentOrderId !== 'string' || !fulfillmentOrderId.match(/^[0-9]+?$/)) {
                throw new exceptionalError(`Fulfillment order IDs must be numeric (but as a string.. yeah)`);
            }
        }
        if (!trackingDetails || shopUtils.isValEmpty(trackingDetails.carrier) || shopUtils.isValEmpty(trackingDetails.trackingNumber)) {
            throw new exceptionalError(`Carrier and tracking number are mandatory fields`);
        }
        fulfillmentUpdate.line_items_by_fulfillment_order = [];
        for (const id of fulfillmentOrderIds) {
            fulfillmentUpdate.line_items_by_fulfillment_order.push({ fulfillment_order_id: Number(id) });
        }
        fulfillmentUpdate.tracking_info = {
            number: trackingDetails.trackingNumber,
            company: trackingDetails.carrier
        };
        if (!shopUtils.isValEmpty(trackingDetails.url)) {
            if (!trackingDetails.url.match(/^http.+?\.\w{2,4}.*?$/i)) {
                throw new exceptionalError(`Invalid tracking url`);
            }
            fulfillmentUpdate.tracking_info.url = trackingDetails.url;
        }
        fulfillmentUpdate.notify_customer = notifyUser;
        yield shopAPI.submitObjectToAPI(fulfillmentUpdate);
    });
}
export function closeOrder(orderId) {
    return __awaiter(this, void 0, void 0, function* () {
        const order = new shopAPI.shopify.rest.Order({ session: shopAPI.shopSession });
        order.id = Number(orderId);
        yield order.close({});
    });
}
export function createOrder(niceNewOrder, shippingMethod) {
    return __awaiter(this, void 0, void 0, function* () {
        let newShopOrder = shopConverter.niceOrderToShopOrder(niceNewOrder, shippingMethod, shopConstants.sendEmails, shopConstants.tagFromSF);
        yield shopAPI.submitObjectToAPI(newShopOrder);
        return String(newShopOrder.id);
    });
}
