//@ts-nocheck
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import * as shopConversions from '../shopify.conversions.js';
import * as shopUtils from '../shopify.utils.js';
import * as shopConstants from '../shopify.constants.js';
import chai from 'chai';
import chaiAsPromised from "chai-as-promised";
const expect = chai.expect;
chai.use(chaiAsPromised);
import * as dotenv from 'dotenv';
dotenv.config();
const isProduction = (process.env.NODE_ENV === 'PRODUCTION');
describe('Convert nice objects to shopify objects', () => __awaiter(void 0, void 0, void 0, function* () {
    let testOrder = {
        currency: "USD",
        orderNumber: "someOrder1234",
        isTest: false,
        address: {
            firstName: "Chris",
            lastName: "Nevin",
            addressOne: "56 Brodie St",
            city: "New York",
            province: "NY",
            country: "US",
            email: "some@thing.com",
            phone: "13657940654",
            postalCode: "12345"
        },
        source: shopConstants.sourceTypes.shopify,
        items: [
            {
                qty: 5,
                variantId: "123456",
                unitPrice: 50,
                title: "someThing",
                variantTitle: "someVariantTitle"
            }
        ],
        requestedShipping: "Express",
        money: {
            subTotal: 0,
            tax: 0,
            shipping: 0,
            discount: 0,
            total: 0
        }
    };
    context('Validations and checks', () => __awaiter(void 0, void 0, void 0, function* () {
        it('Validates line items', () => __awaiter(void 0, void 0, void 0, function* () {
            for (const field of shopConstants.mandatoryLineItemFields) {
                const oldVal = testOrder.items[0][field];
                testOrder.items[0][field] = null;
                expect(() => { shopConversions.niceOrderToShopOrder(testOrder, shopConstants.shipSpeedStandard, false); }).to.throw(`Line Item field ${field} is mandatory`);
                testOrder.items[0][field] = oldVal;
            }
            const oldLineitems = testOrder.items;
            testOrder.items = [];
            expect(() => { shopConversions.niceOrderToShopOrder(testOrder, shopConstants.shipSpeedStandard, false); }).to.throw(`No items provided`);
            testOrder.items = oldLineitems;
        }));
        it('Validates addresses', () => __awaiter(void 0, void 0, void 0, function* () {
            for (const field of shopConstants.mandatoryAddressFields) {
                const oldVal = testOrder.address[field];
                testOrder.address[field] = null;
                expect(() => { shopConversions.niceOrderToShopOrder(testOrder, shopConstants.shipSpeedStandard, false); }).to.throw(`Address field ${field} is mandatory`);
                testOrder.address[field] = oldVal;
            }
            testOrder.address.country = "USA";
            expect(() => { shopConversions.niceOrderToShopOrder(testOrder, shopConstants.shipSpeedStandard, false); }).to.throw(`Country and province codes must be two character ISO standard`);
            testOrder.address.country = "US";
            testOrder.address.province = "NYC";
            expect(() => { shopConversions.niceOrderToShopOrder(testOrder, shopConstants.shipSpeedStandard, false); }).to.throw(`Country and province codes must be two character ISO standard`);
            testOrder.address.province = "NY";
        }));
        it('Validates other mandatory fields', () => __awaiter(void 0, void 0, void 0, function* () {
            const oldOrderNumber = testOrder.orderNumber;
            delete testOrder.orderNumber;
            expect(() => { shopConversions.niceOrderToShopOrder(testOrder, shopConstants.shipSpeedStandard, false); }).to.throw('Top Level field orderNumber is mandatory');
            testOrder.orderNumber = oldOrderNumber;
            const oldCurrency = testOrder.currency;
            testOrder.currency = null;
            expect(() => { shopConversions.niceOrderToShopOrder(testOrder, shopConstants.shipSpeedStandard, false); }).to.throw('Top Level field currency is mandatory');
            testOrder.currency = "FSADAD";
            expect(() => { shopConversions.niceOrderToShopOrder(testOrder, shopConstants.shipSpeedStandard, false); }).to.throw(`Invalid currency FSADAD`);
            testOrder.currency = "USD";
        }));
        it('Validates monies', () => __awaiter(void 0, void 0, void 0, function* () {
            testOrder.money.subTotal = null;
            expect(() => { shopConversions.niceOrderToShopOrder(testOrder, shopConstants.shipSpeedStandard, false); }).to.throw(`Subtotal mismatch - line items totaled 250 but subTotal was 0`);
            testOrder.money.subTotal = "asdad";
            expect(() => { shopConversions.niceOrderToShopOrder(testOrder, shopConstants.shipSpeedStandard, false); }).to.throw(`Money field subTotal must be a number`);
            testOrder.money.subTotal = 1;
            testOrder.money.shipping = 2;
            testOrder.money.discount = -1;
            testOrder.money.tax = 10;
            testOrder.money.total = 13;
            expect(() => { shopConversions.niceOrderToShopOrder(testOrder, shopConstants.shipSpeedStandard, false); }).to.throw(`Discount cannot be negative`);
            testOrder.money.discount = 25.50;
            expect(() => { shopConversions.niceOrderToShopOrder(testOrder, shopConstants.shipSpeedStandard, false); }).to.throw(`Subtotal mismatch - line items totaled 250 but subTotal was 1`);
            // this should be in its own test but MEH 
            expect(shopUtils.roundCashToString(150.264)).to.equal("150.26");
            expect(shopUtils.roundCashToString(99.5750001)).to.equal("99.58");
            expect(shopUtils.roundCashToString(125)).to.equal("125.00");
            testOrder.money.subTotal = 250;
            expect(() => { shopConversions.niceOrderToShopOrder(testOrder, shopConstants.shipSpeedStandard, false); }).to.throw(`Grand Total Mismatch (13 != 236.5`);
            testOrder.money.total = 236.50001;
        }));
        it('Validates the other function args', () => __awaiter(void 0, void 0, void 0, function* () {
            expect(() => { shopConversions.niceOrderToShopOrder(testOrder, "asdadsad", false); }).to.throw(`Invalid shipping service asdadsad`);
            expect(() => { shopConversions.niceOrderToShopOrder(testOrder, null, false); }).to.throw(`Invalid shipping service `);
            expect(() => { shopConversions.niceOrderToShopOrder(testOrder, shopConstants.shipSpeedPriority, 5); }).to.throw(`notifyUser must be boolean`);
        }));
    }));
    context('Creates a nice new shop order object', () => __awaiter(void 0, void 0, void 0, function* () {
        it('Works', () => __awaiter(void 0, void 0, void 0, function* () {
            const notifyUser = true;
            let goodShopObj = shopConversions.niceOrderToShopOrder(testOrder, shopConstants.shipSpeedStandard, notifyUser);
            // Top Level
            expect(goodShopObj.name).to.equal(testOrder.orderNumber);
            expect(goodShopObj.currency).to.equal(testOrder.currency);
            expect(goodShopObj.test).to.equal(testOrder.isTest);
            expect(goodShopObj.email).to.equal(testOrder.address.email);
            expect(goodShopObj.fulfillment_status).to.equal(shopConstants.fulfillmentOutboundStatusNew);
            expect(goodShopObj.send_receipt).to.be.true;
            expect(goodShopObj.send_fulfillment_receipt).to.be.true;
            // Money
            expect(goodShopObj.total_price).to.equal(shopUtils.roundCashToString(testOrder.money.total));
            expect(goodShopObj.total_discounts).to.equal(shopUtils.roundCashToString(testOrder.money.discount));
            expect(goodShopObj.total_tax).to.equal(shopUtils.roundCashToString(testOrder.money.tax));
            expect(goodShopObj.total_line_items_price).to.equal(shopUtils.roundCashToString(testOrder.money.subTotal));
            expect(goodShopObj.total_shipping_price_set).to.deep.equal({
                shop_money: {
                    currency_code: testOrder.currency,
                    amount: shopUtils.roundCashToString(testOrder.money.shipping)
                },
                presentment_money: {
                    currency_code: testOrder.currency,
                    amount: shopUtils.roundCashToString(testOrder.money.shipping)
                }
            });
            // Line Items
            expect(goodShopObj.line_items[0].quantity).to.equal(testOrder.items[0].qty);
            expect(goodShopObj.line_items[0].variant_id).to.equal(Number(testOrder.items[0].variantId));
            expect(goodShopObj.line_items[0].price).to.equal(shopUtils.roundCashToString(testOrder.items[0].unitPrice));
            // Shipping Lines
            expect(goodShopObj.shipping_lines[0].custom).to.be.true;
            expect(goodShopObj.shipping_lines[0].price).to.equal(shopUtils.roundCashToString(testOrder.money.shipping));
            expect(goodShopObj.shipping_lines[0].title).to.equal(shopConstants.shipSpeedStandard);
            // Address
            let expectedAddress = {
                first_name: testOrder.address.firstName,
                last_name: testOrder.address.lastName,
                address1: testOrder.address.addressOne,
                city: testOrder.address.city,
                country_code: testOrder.address.country,
                province_code: testOrder.address.province,
                zip: testOrder.address.postalCode,
                name: `${testOrder.address.firstName} ${testOrder.address.lastName}`,
                phone: testOrder.address.phone
            };
            expect(goodShopObj.shipping_address).to.deep.equal(expectedAddress);
            testOrder.address.addressTwo = "fasdadafq";
            testOrder.address.company = "incCo";
            goodShopObj = shopConversions.niceOrderToShopOrder(testOrder, shopConstants.shipSpeedStandard, notifyUser);
            expectedAddress.address2 = testOrder.address.addressTwo;
            expectedAddress.company = testOrder.address.company;
            expect(goodShopObj.shipping_address).to.deep.equal(expectedAddress);
        }));
    }));
}));
